@@ -4,22 +4,14 @@
 /// "Use Terms").   Any redistribution of this code must retain the above 
 /// copyright and this notice and otherwise comply with the Use Terms.
 /**
- * @path ch15/15.2/15.2.3/15.2.3.7/15.2.3.7-5-b-129.js
- * @description Object.defineProperties - 'descObj' is a Number object which implements its own [[Get]] method to get 'value' property (8.10.5 step 5.a)
+ * @path ch15/15.2/15.2.3/15.2.3.8/15.2.3.8-0-2.js
+ * @description Object.seal must exist as a function taking 1 parameter
  */
 
 
 function testcase() {
-        var obj = {};
-
-        var descObj = new Number(-9);
-
-        descObj.value = "Number";
-
-        Object.defineProperties(obj, {
-            property: descObj
-        });
-
-        return obj.property === "Number";
-    }
+  if (Object.seal.length === 1) {
+    return true;
+  }
+ }
 runTestCase(testcase);
